@@ -2,14 +2,14 @@
 # Creation date: 2003-03-30 15:23:31
 # Authors: Don
 # Change log:
-# $Id: Statement.pm,v 1.2 2004/07/01 06:37:12 don Exp $
+# $Id: Statement.pm,v 1.4 2004/10/30 00:09:31 don Exp $
 
 use strict;
 
 {   package DBIx::Wrapper::Statement;
 
     use vars qw($VERSION);
-    $VERSION = do { my @r=(q$Revision: 1.2 $=~/\d+/g); sprintf "%d."."%02d"x$#r,@r };
+    $VERSION = do { my @r=(q$Revision: 1.4 $=~/\d+/g); sprintf "%d."."%02d"x$#r,@r };
 
     sub new {
         my ($proto) = @_;
@@ -40,6 +40,26 @@ use strict;
         my ($self, $parent) = @_;
         $$self{_parent} = $parent;
     }
+
+    sub _getQuery {
+        my $self = shift;
+        return $self->{_query};
+    }
+
+    sub _setQuery {
+        my $self = shift;
+        my $query = shift;
+        $self->{_query} = $query;
+    }
+
+    sub _getRequestObj {
+        return shift()->{_request_obj};
+    }
+    
+    sub _setRequestObj {
+        my $self = shift;
+        $self->{_request_obj} = shift;
+    }
     
 }
 
@@ -73,6 +93,6 @@ DBIx::Wrapper::Statement - Statement wrapper used by DBIx::Wrapper
 
 =head1 VERSION
 
-$Id: Statement.pm,v 1.2 2004/07/01 06:37:12 don Exp $
+$Id: Statement.pm,v 1.4 2004/10/30 00:09:31 don Exp $
 
 =cut
